@@ -11,17 +11,19 @@ use strict;
 use AutoLoader 'AUTOLOAD';
 use Apache::Session;
 use base qw(Apache::Session);
+use Lemonldap::NG::Common::Apache::Session::Serialize::JSON;
 use Lemonldap::NG::Common::Apache::Session::Store;
 use Lemonldap::NG::Common::Apache::Session::Lock;
 
-our $VERSION = '1.4.9';
+our $VERSION = '1.9.0';
 
 sub _load {
-    my $backend = shift;
+    my ( $backend, $func ) = @_;
     unless ( $backend->can('populate') ) {
         eval "require $backend";
         die $@ if ($@);
     }
+    return $func ? $backend->can($func) : 0;
 }
 
 sub populate {
@@ -33,6 +35,15 @@ sub populate {
         no strict 'refs';
         $self = $self->$backend(@_);
     }
+    if ( $backend =~
+/^Apache::Session::(?:(?:Postgre|Redi)s|(?:Oracl|Sybas)e|(?:My|No)SQL|F(?:ile|lex)|Cassandra|LDAP)/
+        and !$self->{args}->{useStorable} )
+    {
+        $self->{serialize} =
+          \&Lemonldap::NG::Common::Apache::Session::Serialize::JSON::serialize;
+        $self->{unserialize} =
+          \&Lemonldap::NG::Common::Apache::Session::Serialize::JSON::unserialize;
+    }
     if ( $self->{args}->{generateModule} ) {
         my $generate = $self->{args}->{generateModule};
         eval "require $generate";
@@ -46,14 +57,10 @@ sub populate {
     }
 
     # If cache is configured, use our specific object store module
-    if ( $self->{args}->{localStorage} ) {
+    if ( $> and $self->{args}->{localStorage} ) {
         $self->{args}->{object_store} = $self->{object_store};
         $self->{object_store} =
           Lemonldap::NG::Common::Apache::Session::Store->new($self);
-    }
-
-    # If cache is configured, use our specific lock_manager object module
-    if ( $self->{args}->{localStorage} ) {
         $self->{args}->{lock_manager} = $self->{lock_manager};
         $self->{lock_manager} =
           Lemonldap::NG::Common::Apache::Session::Lock->new($self);
@@ -70,11 +77,8 @@ sub setId {
 
 sub searchOn {
     my ( $class, $args, $selectField, $value, @fields ) = splice @_;
-    my $backend = $args->{backend};
-    _load($backend);
-    if ( $backend->can('searchOn') ) {
-        return $backend->searchOn( $args, $selectField, $value, @fields );
-    }
+    return $args->{backend}->searchOn( $args, $selectField, $value, @fields )
+      if ( _load( $args->{backend}, 'searchOn' ) );
     my %res = ();
     $class->get_key_from_all_sessions(
         $args,
@@ -96,11 +100,9 @@ sub searchOn {
 
 sub searchOnExpr {
     my ( $class, $args, $selectField, $value, @fields ) = splice @_;
-    my $backend = $args->{backend};
-    _load($backend);
-    if ( $backend->can('searchOnExpr') ) {
-        return $backend->searchOnExpr( $args, $selectField, $value, @fields );
-    }
+    return $args->{backend}
+      ->searchOnExpr( $args, $selectField, $value, @fields )
+      if ( _load( $args->{backend}, 'searchOnExpr' ) );
     $value = quotemeta($value);
     $value =~ s/\\\*/\.\*/g;
     $value = qr/^$value$/;
@@ -125,6 +127,8 @@ sub searchOnExpr {
 
 sub searchLt {
     my ( $class, $args, $selectField, $value, @fields ) = splice @_;
+    return $args->{backend}->searchLt( $args, $selectField, $value, @fields )
+      if ( _load( $args->{backend}, 'searchLt' ) );
     my %res = ();
     $class->get_key_from_all_sessions(
         $args,
@@ -147,11 +151,18 @@ sub searchLt {
 sub get_key_from_all_sessions {
     my $class = shift;
 
-    #my ( $class, $args, $data ) = splice @_;
-    my $backend = $_[0]->{backend};
-    _load($backend);
-    if ( $backend->can('get_key_from_all_sessions') ) {
-        return $backend->get_key_from_all_sessions(@_);
+    my ( $args, $data ) = @_;
+    my $backend = $args->{backend};
+    if ( _load( $backend, 'get_key_from_all_sessions' ) ) {
+        return $backend->get_key_from_all_sessions( $args, $data );
+    }
+    if ( $args->{useStorable} ) {
+        require Storable;
+        $args->{unserialize} = \&Storable::thaw;
+    }
+    else {
+        $args->{unserialize} =
+          \&Lemonldap::NG::Common::Apache::Session::Serialize::JSON::_unserialize;
     }
     if ( $backend =~
 /^Apache::Session::(MySQL|MySQL::NoLock|Postgres|Oracle|Sybase|Informix)$/
@@ -173,7 +184,6 @@ sub get_key_from_all_sessions {
 }
 
 sub decodeThaw64 {
-    require Storable;
     require MIME::Base64;
     my $s = shift;
     return Storable::thaw( MIME::Base64::decode_base64($s) );
@@ -181,13 +191,15 @@ sub decodeThaw64 {
 
 sub _dbiGKFAS {
     my ( $class, $type, $args, $data ) = @_;
-    my $unserialize;
-    require Storable;
-    if ( $type =~ /(?:MySQL)/ ) {
-        $unserialize = \&Storable::thaw;
-    }
-    else {
-        $unserialize = \&decodeThaw64;
+    my $next;
+    if ( $type !~ /(?:MySQL)/ ) {
+        $next = sub {
+            require MIME::Base64;
+            return thaw( MIME::Base64::decode_base64( $_[0] ) );
+        };
+        if ( $args->{useStorable} ) {
+            $args->{unserialize} = $next;
+        }
     }
 
     my $dbh =
@@ -198,16 +210,17 @@ sub _dbiGKFAS {
     my %res;
     while ( my @row = $sth->fetchrow_array ) {
         if ( ref($data) eq 'CODE' ) {
-            my $tmp = &$data( $unserialize->( $row[1] ), $row[0] );
+            my $tmp =
+              &$data( $args->{unserialize}->( $row[1], $next ), $row[0] );
             $res{ $row[0] } = $tmp if ( defined($tmp) );
         }
         elsif ($data) {
             $data = [$data] unless ( ref($data) );
-            my $tmp = $unserialize->( $row[1] );
+            my $tmp = $args->{unserialize}->( $row[1], $next );
             $res{ $row[0] }->{$_} = $tmp->{$_} foreach (@$data);
         }
         else {
-            $res{ $row[0] } = $unserialize->( $row[1] );
+            $res{ $row[0] } = $args->{unserialize}->( $row[1], $next );
         }
     }
     return \%res;
@@ -216,7 +229,6 @@ sub _dbiGKFAS {
 sub _FileGKFAS {
     my ( $class, $args, $data ) = @_;
     $args->{Directory} ||= '/tmp';
-    require Storable;
 
     unless ( opendir DIR, $args->{Directory} ) {
         die "Cannot open directory $args->{Directory}\n";
@@ -229,7 +241,7 @@ sub _FileGKFAS {
         open F, "$args->{Directory}/$f";
         my $row = join '', <F>;
         if ( ref($data) eq 'CODE' ) {
-            eval { $res{$f} = &$data( Storable::thaw($row), $f ); };
+            eval { $res{$f} = &$data( $args->{unserialize}->($row), $f ); };
             if ($@) {
                 $res{$f} = &$data( undef, $f );
             }
@@ -237,7 +249,7 @@ sub _FileGKFAS {
         elsif ($data) {
             $data = [$data] unless ( ref($data) );
             my $tmp;
-            eval { $tmp = Storable::thaw($row); };
+            eval { $tmp = $args->{unserialize}->($row); };
             if ($@) {
                 $res{$f}->{$_} = undef foreach (@$data);
             }
@@ -246,7 +258,7 @@ sub _FileGKFAS {
             }
         }
         else {
-            eval { $res{$f} = Storable::thaw($row); };
+            eval { $res{$f} = $args->{unserialize}->($row); };
         }
     }
     return \%res;
@@ -286,7 +298,6 @@ sub _PHPGKFAS {
 
 sub _DBFileGKFAS {
     my ( $class, $args, $data ) = @_;
-    require Storable;
 
     if ( !tied %{ $class->{dbm} } ) {
         my $rv = tie %{ $class->{dbm} }, 'DB_File', $args->{FileName};
@@ -298,15 +309,16 @@ sub _DBFileGKFAS {
     my %res;
     foreach my $k ( keys %{ $class->{dbm} } ) {
         if ( ref($data) eq 'CODE' ) {
-            $res{$k} = &$data( Storable::thaw( $class->{dbm}->{$k} ), $k );
+            $res{$k} =
+              &$data( $args->{unserialize}->( $class->{dbm}->{$k} ), $k );
         }
         elsif ($data) {
             $data = [$data] unless ( ref($data) );
-            my $tmp = Storable::thaw( $class->{dbm}->{$k} );
+            my $tmp = $args->{unserialize}->( $class->{dbm}->{$k} );
             $res{$k}->{$_} = $tmp->{$_} foreach (@$data);
         }
         else {
-            $res{$k} = Storable::thaw( $class->{dbm}->{$k} );
+            $res{$k} = $args->{unserialize}->( $class->{dbm}->{$k} );
         }
     }
     return \%res;
@@ -314,20 +326,28 @@ sub _DBFileGKFAS {
 
 sub _LDAPGKFAS {
     my ( $class, $args, $data ) = @_;
+    $args->{ldapObjectClass}      ||= 'applicationProcess';
+    $args->{ldapAttributeId}      ||= 'cn';
+    $args->{ldapAttributeContent} ||= 'description';
 
     my $ldap = Apache::Session::Store::LDAP::ldap( { args => $args } );
     my $msg = $ldap->search(
         base   => $args->{ldapConfBase},
-        filter => '(objectClass=applicationProcess)',
-        attrs  => [ 'cn', 'description' ],
+        filter => '(objectClass=' . $args->{ldapObjectClass} . ')',
+        attrs  => [ $args->{ldapAttributeId}, $args->{ldapAttributeContent} ],
     );
+
     $ldap->unbind();
-    $ldap->disconnect();
+
     Apache::Session::Store::LDAP->logError($msg) if ( $msg->code );
+
     my %res;
+
     foreach my $entry ( $msg->entries ) {
-        my ( $k, $v ) =
-          ( $entry->get_value('cn'), $entry->get_value('description') );
+        my ( $k, $v ) = (
+            $entry->get_value('cn'),
+            $entry->get_value( $args->{ldapAttributeContent} )
+        );
         eval { $v = decodeThaw64($v); };
         next if ($@);
         if ( ref($data) eq 'CODE' ) {
