@@ -368,7 +368,7 @@ sub extract_lang {
         $lang = ( split /-/, $lang )[0];
 
         # Go to next if lang was already added
-        next if grep( /$lang/, @res );
+        next if grep( /\Q$lang\E/, @res );
 
         # Store lang only if size is 2 characters
         push @res, $lang if ( length($lang) == 2 );
@@ -529,11 +529,11 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =over
 
-=item Copyright (C) 2008, 2009, 2010 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+=item Copyright (C) 2008-2016 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
-=item Copyright (C) 2012, 2013 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item Copyright (C) 2012-2013 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
-=item Copyright (C) 2010, 2011, 2012 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+=item Copyright (C) 2010-2016 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
 =back
 
