@@ -8,9 +8,12 @@
 package Lemonldap::NG::Common::Conf;
 
 use strict;
+use utf8;
 no strict 'refs';
-use Lemonldap::NG::Common::Conf::Constants;     #inherits
-use Lemonldap::NG::Common::Conf::Attributes;    #inherits
+use Lemonldap::NG::Common::Conf::Constants;    #inherits
+
+# TODO: don't import this big file, use a proxy
+use Lemonldap::NG::Common::Conf::DefaultValues;    #inherits
 use Lemonldap::NG::Common::Crypto
   ;    #link protected cipher Object "cypher" in configuration hash
 use Config::IniFiles;
@@ -20,8 +23,8 @@ use Config::IniFiles;
 #inherits Lemonldap::NG::Common::Conf::SOAP
 #inherits Lemonldap::NG::Common::Conf::LDAP
 
-our $VERSION = '1.4.1';
-our $msg;
+our $VERSION = '1.9.0';
+our $msg     = '';
 our $iniObj;
 
 BEGIN {
@@ -92,6 +95,9 @@ sub new {
         if ($@) {
             $msg .= "Unable to load $self->{localStorage}: $@.\n";
         }
+
+        # TODO: defer that until $> > 0 (to avoid creating local cache with
+        # root privileges
         else {
             $self->{refLocalStorage} =
               $self->{localStorage}->new( $self->{localStorageOptions} );
@@ -100,21 +106,22 @@ sub new {
     return $self;
 }
 
-## @method int saveConf(hashRef conf)
+## @method int saveConf(hashRef conf, hash args)
 # Serialize $conf and call store().
 # @param $conf Lemonldap::NG configuration hashRef
-# @return Number of the saved configuration, 0 if case of error.
+# @param %args Parameters
+# @return Number of the saved configuration, 0 in case of error.
 sub saveConf {
-    my ( $self, $conf ) = @_;
+    my ( $self, $conf, %args ) = @_;
 
     my $last = $self->lastCfg;
 
     # If configuration was modified, return an error
-    if ( not $self->{force} ) {
+    if ( not $args{force} ) {
         return CONFIG_WAS_CHANGED if ( $conf->{cfgNum} != $last );
         return DATABASE_LOCKED if ( $self->isLocked() or not $self->lock() );
     }
-    $conf->{cfgNum} = $last + 1 unless ( $self->{cfgNumFixed} );
+    $conf->{cfgNum} = $last + 1 unless ( $args{cfgNumFixed} );
     delete $conf->{cipher};
 
     # Try to store configuration
@@ -141,8 +148,9 @@ sub saveConf {
 sub getConf {
     my ( $self, $args ) = @_;
 
-    # Use only cache to get conf
-    if (    $args->{local}
+    # Use only cache to get conf if $args->{local} is set
+    if (    $>
+        and $args->{local}
         and ref( $self->{refLocalStorage} )
         and my $res = $self->{refLocalStorage}->get('conf') )
     {
@@ -163,62 +171,75 @@ sub getConf {
             $r = $self->getDBConf($args);
         }
         else {
-            eval { $r = $self->{refLocalStorage}->get('conf') };
+            eval { $r = $self->{refLocalStorage}->get('conf') }
+              if ( $> and not $args->{noCache} );
             $msg = "Warn: $@" if ($@);
-            if ( ref($r) and $r->{cfgNum} == $args->{cfgNum} ) {
+            if (    ref($r)
+                and $r->{cfgNum}
+                and $args->{cfgNum}
+                and $r->{cfgNum} == $args->{cfgNum} )
+            {
                 $msg .=
                   "Configuration unchanged, get configuration from cache.\n";
                 $args->{noCache} = 1;
             }
             else {
                 $r = $self->getDBConf($args);
-            }
-        }
+                return undef unless ( $r->{cfgNum} );
 
-        # Get default values
-        my $confAttributes = Lemonldap::NG::Common::Conf::Attributes->new();
+                # TODO: default values may not be set here
+                unless ( $args->{raw} ) {
 
-        my @attributes = $confAttributes->meta()->get_attribute_list();
+                    # Adapt some values before storing in local cache
+                    # Get default values
+                    my $defaultValues =
+                      Lemonldap::NG::Common::Conf::DefaultValues
+                      ->defaultValues();
+
+                    foreach my $k ( keys %$defaultValues ) {
+                        $r->{$k} //= $defaultValues->{$k};
+                    }
+                }
+
+                # Convert old option useXForwardedForIP into trustedProxies
+                if ( defined $r->{useXForwardedForIP}
+                    and $r->{useXForwardedForIP} == 1 )
+                {
+                    $r->{trustedProxies} = '*';
+                    delete $r->{useXForwardedForIP};
+                }
+
+                # Force Choice backend
+                if ( $r->{authentication} eq "Choice" ) {
+                    $r->{userDB}     = "Choice";
+                    $r->{passwordDB} = "Choice";
+                }
+
+            # Some parameters expect key name (example), not variable ($example)
+                foreach (qw/whatToTrace/) {
+                    if ( defined $r->{$_} ) {
+                        $r->{$_} =~ s/^\$//;
+                    }
+                }
+
+                # Store modified configuration in cache
+                $self->setLocalConf($r)
+                  if ( $self->{refLocalStorage}
+                    and not( $args->{noCache} or $args->{raw} ) );
 
-        foreach my $name (@attributes) {
-            unless ( defined $r->{$name} ) {
-                $r->{$name} = $confAttributes->$name;
             }
         }
 
         # Create cipher object
-        eval { $r->{cipher} = Lemonldap::NG::Common::Crypto->new( $r->{key} ); };
-        if ($@) {
-            $msg .= "Bad key: $@. \n";
-            return $r;
-        }
-
-        # Adapt some values
-
-        # Convert old option useXForwardedForIP into trustedProxies
-        if ( defined $r->{useXForwardedForIP}
-            and $r->{useXForwardedForIP} == 1 )
-        {
-            $r->{trustedProxies} = '*';
-        }
-
-        # Force Choice backend
-        if ( $r->{authentication} eq "Choice" ) {
-            $r->{userDB}     = "Choice";
-            $r->{passwordDB} = "Choice";
-        }
-
-        # Some parameters expect key name (example), not variable ($example)
-        foreach (qw/whatToTrace/) {
-            if ( defined $r->{$_} ) {
-                $r->{$_} =~ s/^\$//;
+        unless ( $args->{raw} ) {
+            eval {
+                $r->{cipher} = Lemonldap::NG::Common::Crypto->new( $r->{key} );
+            };
+            if ($@) {
+                $msg .= "Bad key: $@. \n";
             }
         }
 
-        # Store modified configuration in cache
-        $self->setLocalConf($r)
-          if ( $self->{refLocalStorage} and not( $args->{noCache} ) );
-
         # Return configuration hash
         return $r;
     }
@@ -278,7 +299,6 @@ sub getLocalConf {
             return $r;
         }
     }
-    $self->{_iniObj} = $cfg;
 
     # First load all default section parameters
     if ($loaddefault) {
@@ -320,6 +340,7 @@ sub getLocalConf {
 # @param $conf Lemonldap::NG configuration hashRef
 sub setLocalConf {
     my ( $self, $conf ) = @_;
+    return unless ($>);
     eval { $self->{refLocalStorage}->set( "conf", $conf ) };
     $msg .= "Warn: $@\n" if ($@);
 }
@@ -341,9 +362,12 @@ sub getDBConf {
           : $a[0];
     }
     my $conf = $self->load( $args->{cfgNum} );
-    $msg .= "Get configuration $conf->{cfgNum}.\n";
+    $msg .= "Get configuration $conf->{cfgNum}.\n"
+      if ( defined $conf->{cfgNum} );
     $self->setLocalConf($conf)
-      if ( $self->{refLocalStorage} and not( $args->{noCache} ) );
+      if (  ref($conf)
+        and $self->{refLocalStorage}
+        and not( $args->{noCache} ) );
     return $conf;
 }
 
@@ -567,13 +591,13 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =over
 
-=item Copyright (C) 2008, 2009, 2010 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+=item Copyright (C) 2008-2016 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
 =item Copyright (C) 2012 by Sandro Cazzaniga, E<lt>cazzaniga.sandro@gmail.comE<gt>
 
 =item Copyright (C) 2012 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
-=item Copyright (C) 2009, 2010, 2011, 2012, 2013 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+=item Copyright (C) 2009-2016 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
 =back
 
