@@ -1,33 +1,53 @@
 package Lemonldap::NG::Common::Conf::RDBI;
 
 use strict;
+use utf8;
 use Lemonldap::NG::Common::Conf::Serializer;
 use Lemonldap::NG::Common::Conf::_DBI;
 
-our $VERSION = '1.2.2_01';
+our $VERSION = '1.4.0';
 our @ISA     = qw(Lemonldap::NG::Common::Conf::_DBI);
 
 sub store {
     my ( $self, $fields ) = @_;
+    my $cfgNum = $fields->{cfgNum};
     $self->{noQuotes} = 1;
     $fields = $self->serialize($fields);
-    my $errors = 0;
+
+    my $req;
+    my $lastCfg = $self->lastCfg;
+
+    $self->_dbh->begin_work;
+    eval { $self->_dbh->do("BEGIN") };
+    if ( $lastCfg == $cfgNum ) {
+        $req = $self->_dbh->prepare(
+"UPDATE $self->{dbiTable} SET field=?, value=? WHERE cfgNum=? AND field=?"
+        );
+
+    }
+    else {
+        $req = $self->_dbh->prepare(
+            "INSERT INTO $self->{dbiTable} (cfgNum,field,value) VALUES (?,?,?)"
+        );
+    }
+    unless ($req) {
+        $self->logError;
+        return UNKNOWN_ERROR;
+    }
     while ( my ( $k, $v ) = each %$fields ) {
-        my $tmp =
-          $self->_dbh->prepare( "insert into "
-              . $self->{dbiTable}
-              . " (cfgNum,field,value) values (?,?,?)" );
-        unless ( $tmp and $tmp->execute( $fields->{cfgNum}, $k, $v ) ) {
+        my @execValues;
+        if ( $lastCfg == $cfgNum ) {
+            @execValues = ( $k, $v, $cfgNum, $k );
+        }
+        else { @execValues = ( $cfgNum, $k, $v ); }
+        unless ( $req->execute(@execValues) ) {
             $self->logError;
-            $errors++;
-            last;
+            $self->_dbh->do("ROLLBACK");
+            return UNKNOWN_ERROR;
         }
     }
-    eval { $errors ? $self->_dbh->do("ROLLBACK") : $self->_dbh->do("COMMIT"); };
-    unless ( $self->unlock ) {
-        $self->logError;
-    }
-    return $errors ? UNKNOWN_ERROR : $fields->{cfgNum};
+    eval { $self->_dbh->do("COMMIT") };
+    return $cfgNum;
 }
 
 sub load {
@@ -36,8 +56,8 @@ sub load {
     my $sth =
       $self->_dbh->prepare( "SELECT cfgNum,field,value from "
           . $self->{dbiTable}
-          . " WHERE cfgNum=$cfgNum" );
-    $sth->execute();
+          . " WHERE cfgNum=?" );
+    $sth->execute($cfgNum);
     my ( $res, @row );
     while ( @row = $sth->fetchrow_array ) {
         $res->{ $row[1] } = $row[2];
