@@ -6,6 +6,7 @@
 package Lemonldap::NG::Common::Conf::SAML::Metadata;
 
 use strict;
+use utf8;
 use warnings;
 use Crypt::OpenSSL::RSA;
 use Crypt::OpenSSL::X509;
@@ -16,7 +17,7 @@ use XML::Simple;
 use Safe;
 use Encode;
 
-our $VERSION = '1.2.2_01';
+our $VERSION = '1.2.1';
 
 ## @cmethod Lemonldap::NG::Common::Conf::SAML::Metadata new(hashRef args)
 # Class constructor.
@@ -58,8 +59,8 @@ sub initializeFromConfHash {
 
     return 0 unless $hash;
 
-    while ( my ( $k, $v ) = each(%$hash) ) {
-        $self->{$k} = $v;
+    foreach my $k ( keys %$hash ) {
+        $self->{$k} = $hash->{$k};
     }
 
     return 1;
@@ -97,8 +98,8 @@ sub initializeFromXML {
 
     # Store data in Metadata object
     if ($data) {
-        while ( my ( $k, $v ) = each( %{$data} ) ) {
-            $self->{$k} = $v;
+        foreach my $k ( keys %{$data} ) {
+            $self->{$k} = $data->{$k};
         }
         return 1;
     }
@@ -111,7 +112,7 @@ sub initializeFromXML {
 # SAML 2 description.
 # @return string
 sub serviceToXML {
-    my ( $self, $file, $conf ) = splice @_;
+    my ( $self, $file, $conf ) = @_;
 
     my $template = HTML::Template->new(
         filename          => "$file",
@@ -375,7 +376,7 @@ sub _toStruct {
 # @param @files Array of filenames
 # @return array of Metadata objects
 sub load {
-    my @files     = splice @_;
+    my @files     = @_;
     my @metadatas = ();
     foreach (@files) {
         my $metadata = new Lemonldap::NG::Common::Conf::SAML::Metadata();
@@ -408,10 +409,10 @@ sub _loadFile {
 # @param conf Configuration hash ref
 # @return value
 sub getValue {
-    my ( $self, $key, $conf ) = splice @_;
+    my ( $self, $key, $conf ) = @_;
 
     # Get portal value
-    my $portal = $conf->{portal} || "http://auth.example.com";
+    my $portal = $conf->{portal} || "http://auth.example.com/";
     $portal =~ s/\/$//;
 
     # Try to get value for the given key in configuraiton
