@@ -1,9 +1,10 @@
 package Lemonldap::NG::Common::Conf::SOAP;
 
 use strict;
+use utf8;
 use SOAP::Lite;
 
-our $VERSION = '1.1.0';
+our $VERSION = '1.4.0';
 
 #parameter proxy Url of SOAP service
 #parameter proxyOptions SOAP::Lite parameters
@@ -64,20 +65,18 @@ sub lastCfg {
     return $self->_soapCall( 'lastCfg', @_ );
 }
 
+# lock and unlock must not be requested by the SOAP client, since
+# they will be done by the SOAP server when storing the config
 sub lock {
-    my $self = shift;
-    return $self->_soapCall( 'lock', @_ );
+    return 1;
 }
 
-# unlock is not needed here since real unlock is called by store
-#sub unlock {
-#    my $self = shift;
-#    return $self->_soapCall( 'unlock', @_ );
-#}
+sub unlock {
+    return 1;
+}
 
 sub isLocked {
-    my $self = shift;
-    return $self->_soapCall( 'isLocked', @_ );
+    return 1;
 }
 
 sub store {
@@ -247,7 +246,7 @@ L<http://lemonldap-ng.org/>
 
 =item Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
-=item Franois-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
 =item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
@@ -267,11 +266,11 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =over
 
-=item Copyright (C) 2008, 2009, 2010 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+=item Copyright (C) 2008-2016 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
-=item Copyright (C) 2012 by Franois-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item Copyright (C) 2012-2014 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
-=item Copyright (C) 2010, 2011, 2012 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+=item Copyright (C) 2010-2012 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
 =back
 
