@@ -1,14 +1,19 @@
 package Lemonldap::NG::Common::Conf::Serializer;
 
-use Data::Dumper;
+use strict;
+use utf8;
+use Encode;
+use JSON;
+use Lemonldap::NG::Common::Conf::Constants;
 
-our $VERSION = '1.2.2_01';
+our $VERSION = '1.9.0';
 
 BEGIN {
-    *Lemonldap::NG::Common::Conf::normalize   = \&normalize;
-    *Lemonldap::NG::Common::Conf::unnormalize = \&unnormalize;
-    *Lemonldap::NG::Common::Conf::serialize   = \&serialize;
-    *Lemonldap::NG::Common::Conf::unserialize = \&unserialize;
+    *Lemonldap::NG::Common::Conf::normalize      = \&normalize;
+    *Lemonldap::NG::Common::Conf::unnormalize    = \&unnormalize;
+    *Lemonldap::NG::Common::Conf::serialize      = \&serialize;
+    *Lemonldap::NG::Common::Conf::unserialize    = \&unserialize;
+    *Lemonldap::NG::Common::Conf::oldUnserialize = \&oldUnserialize;
 }
 
 ## @method string normalize(string value)
@@ -16,7 +21,7 @@ BEGIN {
 # @param value Input value
 # @return normalized string
 sub normalize {
-    my ( $self, $value ) = splice @_;
+    my ( $self, $value ) = @_;
 
     # trim white spaces
     $value =~ s/^\s*(.*?)\s*$/$1/;
@@ -39,7 +44,7 @@ sub normalize {
 # @param value Input value
 # @return unnormalized string
 sub unnormalize {
-    my ( $self, $value ) = splice @_;
+    my ( $self, $value ) = @_;
 
     # Convert simple quotes
     $value =~ s/&#?39;/'/g;
@@ -48,6 +53,9 @@ sub unnormalize {
     $value =~ s/%0D/\r/g;
     $value =~ s/%0A/\n/g;
 
+    # Keep number as numbers
+    $value += 0 if ( $value =~ /^(?:0|(?:\-[0-9]|[1-9])[0-9]*)(?:\.[0-9]+)?$/ );
+
     return $value;
 }
 
@@ -56,29 +64,19 @@ sub unnormalize {
 # @param conf Configuration
 # @return fields
 sub serialize {
-    my ( $self, $conf ) = splice @_;
+    my ( $self, $conf ) = @_;
     my $fields;
 
-    # Data::Dumper options
-    local $Data::Dumper::Indent  = 0;
-    local $Data::Dumper::Varname = "data";
-
     # Parse configuration
-    while ( my ( $k, $v ) = each(%$conf) ) {
+    foreach my $k ( keys %$conf ) {
+        my $v = $conf->{$k};
 
         # 1.Hash ref
         if ( ref($v) ) {
-            $fields->{$k} = $self->normalize( Dumper($v) );
+            $fields->{$k} = to_json($v);
         }
-
-        # 2. Numeric values
-        elsif ( $v =~ /^\d+$/ ) {
-            $fields->{$k} = "$v";
-        }
-
-        # 3. Standard values
         else {
-            $fields->{$k} = $self->normalize($v);
+            $fields->{$k} = $v;
         }
     }
 
@@ -90,7 +88,32 @@ sub serialize {
 # @param fields Fields
 # @return configuration
 sub unserialize {
-    my ( $self, $fields ) = splice @_;
+    my ( $self, $fields ) = @_;
+    my $conf;
+
+    # Parse fields
+    foreach my $k ( keys %$fields ) {
+        my $v = $fields->{$k};
+        if ( $k =~ $hashParameters ) {
+            unless ( utf8::is_utf8($v) ) {
+                $v = encode( 'UTF-8', $v );
+            }
+            $conf->{$k} = ( $v =~ /./ ? eval { from_json($v) } : {} );
+            if ($@) {
+                $Lemonldap::NG::Common::Conf::msg .=
+                  "Unable to decode $k, switching to old format.\n";
+                return $self->oldUnserialize($fields);
+            }
+        }
+        else {
+            $conf->{$k} = $v;
+        }
+    }
+    return $conf;
+}
+
+sub oldUnserialize {
+    my ( $self, $fields ) = @_;
     my $conf;
 
     # Parse fields
@@ -100,50 +123,19 @@ sub unserialize {
         $v =~ s/^'(.*)'$/$1/s;
 
         # Manage hashes
-        if (
-            $k =~ /^(?x:
-	applicationList
-	|authChoiceModules
-	|CAS_proxiedServices
-	|casStorageOptions
-	|exportedHeaders
-	|exportedVars
-	|globalStorageOptions
-        |grantSessionRules
-	|groups
-	|locationRules
-	|logoutServices
-	|macros
-	|notificationStorageOptions
-	|persistentStorageOptions
-	|portalSkinRules
-	|post
-        |reloadUrls
-	|remoteGlobalStorageOptions
-	|samlIDPMetaDataExportedAttributes
-	|samlIDPMetaDataOptions
-	|samlIDPMetaDataXML
-	|samlSPMetaDataExportedAttributes
-	|samlSPMetaDataOptions
-	|samlSPMetaDataXML
-	|samlStorageOptions
-        |sessionDataToRemember
-	|vhostOptions
-	)$/
-            and $v ||= {} and not ref($v)
-          )
-        {
+
+        if ( $k =~ $hashParameters and $v ||= {} and not ref($v) ) {
             $conf->{$k} = {};
 
             # Value should be a Data::Dumper, else this is an old format
             if ( defined($v) and $v !~ /^\$/ ) {
 
-                $msg .=
+                $Lemonldap::NG::Common::Conf::msg .=
 " Warning: configuration is in old format, you've to migrate!";
 
                 eval { require Storable; require MIME::Base64; };
                 if ($@) {
-                    $msg .= " Error: $@";
+                    $Lemonldap::NG::Common::Conf::msg .= " Error: $@";
                     return 0;
                 }
                 $conf->{$k} = Storable::thaw( MIME::Base64::decode_base64($v) );
@@ -159,7 +151,8 @@ sub unserialize {
                 eval $v;
 
                 if ($@) {
-                    $msg .= " Error: cannot read configuration key $k: $@";
+                    $Lemonldap::NG::Common::Conf::msg .=
+                      " Error: cannot read configuration key $k: $@";
                 }
 
                 # Store value in configuration object
