@@ -1,6 +1,7 @@
 package Lemonldap::NG::Common::Conf::_DBI;
 
 use strict;
+use utf8;
 use DBI;
 use Lemonldap::NG::Common::Conf::Constants;    #inherits
 
@@ -10,8 +11,8 @@ our ( @EXPORT, %EXPORT_TAGS );
 
 BEGIN {
     *Lemonldap::NG::Common::Conf::_dbh = \&_dbh;
-    *EXPORT      = \@Lemonldap::NG::Common::Conf::Constants::EXPORT;
-    *EXPORT_TAGS = \%Lemonldap::NG::Common::Conf::Constants::EXPORT_TAGS;
+    push @EXPORT, @Lemonldap::NG::Common::Conf::Constants::EXPORT;
+    %EXPORT_TAGS = %Lemonldap::NG::Common::Conf::Constants::EXPORT_TAGS;
     push @EXPORT,
       qw(prereq available lastCfg _dbh lock isLocked unlock delete logError);
 }
@@ -23,7 +24,8 @@ sub prereq {
           '"dbiChain" is required in *DBI configuration type';
         return 0;
     }
-    print STDERR __PACKAGE__ . 'Warning: "dbiUser" parameter is not set'
+    $Lemonldap::NG::Common::Conf::msg .=
+      'Warning: "dbiUser" parameter is not set'
       unless ( $self->{dbiUser} );
     $self->{dbiTable} ||= "lmConfig";
     1;
@@ -87,8 +89,13 @@ sub unlock {
 
 sub delete {
     my ( $self, $cfgNum ) = @_;
-    $self->_dbh->do( "DELETE from " . $self->{dbiTable} . " WHERE cfgNum=?",
-        {}, $cfgNum );
+    my $req =
+      $self->_dbh->prepare("DELETE FROM $self->{dbiTable} WHERE cfgNum=?");
+    my $res = $req->execute($cfgNum);
+    $Lemonldap::NG::Common::Conf::msg .=
+      "Unable to find conf $cfgNum (" . $self->_dbh->errstr . ")"
+      unless ($res);
+    return $res;
 }
 
 sub logError {
