@@ -7,15 +7,15 @@ package Lemonldap::NG::Common::Notification;
 
 use strict;
 use utf8;
-use Lemonldap::NG::Portal::SharedConf;
 use XML::LibXML;
 use XML::LibXSLT;
 use CGI::Cookie;
+use Scalar::Util 'weaken';
 
 #inherits Lemonldap::NG::Common::Notification::DBI
 #inherits Lemonldap::NG::Common::Notification::File
 
-our $VERSION = '1.2.2_01';
+our $VERSION = '1.9.0';
 our ( $msg, $parser );
 
 ## @cmethod Lemonldap::NG::Common::Notification new(hashref storage)
@@ -23,19 +23,20 @@ our ( $msg, $parser );
 # @param $storage same syntax as Lemonldap::NG::Common::Conf object
 # @return Lemonldap::NG::Common::Notification object
 sub new {
-    my ( $class, $storage ) = splice @_;
+    my ( $class, $storage ) = @_;
     my $self = bless {}, $class;
     (%$self) = (%$storage);
     unless ( $self->{p} ) {
         $msg = "p=>portal is required";
         return 0;
     }
+    weaken $self->{p};
     my $type = $self->{type};
     $self->{type} = "Lemonldap::NG::Common::Notification::$self->{type}"
       unless ( $self->{type} =~ /::/ );
     eval "require $self->{type}";
     if ($@) {
-        $msg = "Error: unknown storage type $type";
+        $msg = "Error: unknown storage type $type ($@)";
         return 0;
     }
     unless ( $self->_prereq ) {
@@ -53,7 +54,7 @@ sub new {
 # @param $mess Text to log
 # @param $level Level (debug|info|notice|error)
 sub lmLog {
-    my ( $self, $mess, $level ) = splice @_;
+    my ( $self, $mess, $level ) = @_;
     $self->{p}->lmLog( "[Notification] $mess", $level );
 }
 
@@ -63,7 +64,7 @@ sub lmLog {
 # @param $portal Lemonldap::NG::Portal object that call
 # @return HTML fragment containing form content
 sub getNotification {
-    my ( $self, $portal ) = splice @_;
+    my ( $self, $portal ) = @_;
     my ( @files, $form );
 
     # Get user datas,
@@ -195,7 +196,7 @@ sub getNotification {
 # @param $portal Lemonldap::NG::Portal object that call
 # @return true if all checkboxes have been checked
 sub checkNotification {
-    my ( $self, $portal ) = splice @_, 0, 2;
+    my ( $self, $portal ) = ( @_, 0, 2 );
     my ( $refs, $checks );
 
     # First, rebuild environment (cookies,...)
@@ -339,7 +340,7 @@ sub checkNotification {
 # @param $xml XML string containing notification
 # @return number of notifications done
 sub newNotification {
-    my ( $self, $xml ) = splice @_;
+    my ( $self, $xml ) = @_;
     eval { $xml = $parser->parse_string($xml); };
     if ($@) {
         $self->lmLog( "Unable to read XML file : $@", 'error' );
@@ -393,7 +394,7 @@ sub newNotification {
 ## @param $myref notification's reference
 ## @return number of deleted notifications
 sub deleteNotification {
-    my ( $self, $uid, $myref ) = splice @_;
+    my ( $self, $uid, $myref ) = @_;
     my @data;
 
     # Check input parameters
@@ -518,6 +519,17 @@ sub _newNotif {
     return &{ $_[0]->{type} . '::newNotif' }(@_);
 }
 
+## @method private string _getIdentifier(string uid, string ref, string date)
+# Get notification identifier
+# @param $uid uid
+# @param $ref ref
+# @param $date date
+# @return the notification identifier
+sub _getIdentifier {
+    no strict 'refs';
+    return &{ $_[0]->{type} . '::getIdentifier' }(@_);
+}
+
 1;
 
 __END__
@@ -542,7 +554,15 @@ L<Lemonldap::NG::Portal>,
 
 =head1 AUTHOR
 
-Xavier Guimard E<lt>x.guimard@free.frE<gt>
+=over
+
+=item Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+
+=item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+
+=item Sandro Cazzaniga, E<lt>cazzaniga.sandro@gmail.comE<gt>
+
+=back
 
 =head1 BUG REPORT
 
@@ -556,11 +576,28 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005, 2007, 2010 by Xavier Guimard E<lt>x.guimard@free.frE<gt>
+=over
+
+=item Copyright (C) 2009-2016 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+
+=item Copyright (C) 2012 by Sandro Cazzaniga, E<lt>cazzaniga.sandro@gmail.comE<gt>
+
+=item Copyright (C) 2010-2012 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+
+=back
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.10.0 or,
-at your option, any later version of Perl 5 you may have available.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see L<http://www.gnu.org/licenses/>.
 
 =cut
 
