@@ -12,7 +12,7 @@ use MIME::Base64;
 use Net::LDAP;
 use utf8;
 
-our $VERSION = '1.3.0';
+our $VERSION = '1.9.0';
 
 ## @method boolean prereq()
 # Check if LDAP parameters are set.
@@ -22,7 +22,7 @@ sub prereq {
     unless ( $self->{ldapServer} ) {
         $self->lmLog( '"ldapServer" is required in LDAP notification type',
             'error' );
-        $Lemonldap::NG::Common::Conf::msg =
+        $Lemonldap::NG::Common::Notification::msg =
           '"ldapServer" is required in LDAP notification type';
         return 0;
     }
@@ -62,9 +62,10 @@ sub get {
         }
         my $xml = $f->{xml};
         utf8::encode($xml);
-        $result->{"$f->{date}#$f->{uid}#$f->{ref}"} = "$xml";
-        $self->lmLog( "notification $f->{date}#$f->{uid}#$f->{ref} found",
-            'info' );
+        my $identifier =
+          &getIdentifier( $self, $f->{uid}, $f->{ref}, $f->{date} );
+        $result->{$identifier} = "$xml";
+        $self->lmLog( "notification $identifier found", 'info' );
 
     }
     return $result;
@@ -109,6 +110,7 @@ sub delete {
     my ( $self, $myref ) = @_;
     my ( $d, $u, $r );
     unless ( ( $d, $u, $r ) = ( $myref =~ /^([^#]+)#(.+?)#(.+)$/ ) ) {
+        $Lemonldap::NG::Common::Notification::msg = "Bad reference $myref";
         $self->lmLog( "Bad reference $myref", 'warn' );
         return 0;
     }
@@ -138,6 +140,7 @@ sub purge {
     my ( $self, $myref, $force ) = @_;
     my ( $d, $u, $r );
     unless ( ( $d, $u, $r ) = ( $myref =~ /^([^#]+)#(.+?)#(.+)$/ ) ) {
+        $Lemonldap::NG::Common::Notification::msg = "Bad reference $myref";
         $self->lmLog( "Bad reference $myref", 'warn' );
         return 0;
     }
@@ -302,8 +305,8 @@ sub _store {
 
     # Store values as {key}value
     my @notifValues;
-    while ( my ( $k, $v ) = each(%$fields) ) {
-        $v = encodeLdapValue($v);
+    foreach my $k ( keys %$fields ) {
+        my $v = encodeLdapValue( $fields->{$k} );
         push @notifValues, "{$k}$v";
     }
 
@@ -417,5 +420,16 @@ sub _ldap {
     return $ldap;
 }
 
+## @method string getIdentifier(string uid, string ref, string date)
+# Get notification identifier
+# @param $uid uid
+# @param $ref ref
+# @param $date date
+# @return the notification identifier
+sub getIdentifier {
+    my ( $self, $uid, $ref, $date ) = @_;
+    return $date . "#" . $uid . "#" . $ref;
+}
+
 1;
 
