@@ -16,7 +16,8 @@ our $VERSION = '1.4.6';
 # Set here all the names of functions that must be available in Safe objects.
 # Not that only functions, not methods, can be written here
 our $functions =
-  [ qw(&checkLogonHours &checkDate &basic &unicode2iso &iso2unicode &groupMatch)
+  [
+    qw(&checkLogonHours &date &checkDate &basic &unicode2iso &iso2unicode &groupMatch)
   ];
 
 ## @function boolean checkLogonHours(string logon_hours, string syntax, string time_correction, boolean default_access)
@@ -27,7 +28,7 @@ our $functions =
 # @param $default_access optional what result to return for users without logons hours
 # @return 1 if access allowed, 0 else
 sub checkLogonHours {
-    my ( $logon_hours, $syntax, $time_correction, $default_access ) = splice @_;
+    my ( $logon_hours, $syntax, $time_correction, $default_access ) = @_;
 
     # Active Directory - logonHours: $attr_src_syntax = octetstring
     # Samba - sambaLogonHours: ???
@@ -70,6 +71,25 @@ sub checkLogonHours {
     return substr( $base2_logon_hours, $hourpos, 1 );
 }
 
+## @function integer date
+# Get current local date
+# @param $gmt optional boolean To return GMT date (default is local date)
+# @return current date on format YYYYMMDDHHMMSS
+sub date {
+    my $gmt = shift;
+    my ( $sec, $min, $hour, $mday, $mon, $year ) = $gmt ? gmtime : localtime;
+
+    $year += 1900;
+    $mon  += 1;
+    $mon  = "0" . $mon  if ( $mon < 10 );
+    $mday = "0" . $mday if ( $mday < 10 );
+    $hour = "0" . $hour if ( $hour < 10 );
+    $min  = "0" . $min  if ( $min < 10 );
+    $sec  = "0" . $sec  if ( $sec < 10 );
+
+    return $year . $mon . $mday . $hour . $min . $sec;
+}
+
 ## @function boolean checkDate(string start, string end, boolean default_access)
 # Function to check a date
 # @param $start string Start date (GMT)
@@ -77,7 +97,7 @@ sub checkLogonHours {
 # @param $default_access optional what result to return for users without start or end start
 # @return 1 if access allowed, 0 else
 sub checkDate {
-    my ( $start, $end, $default_access ) = splice @_;
+    my ( $start, $end, $default_access ) = @_;
 
     # Get date in string
     $start = substr( $start, 0, 14 );
@@ -94,17 +114,7 @@ sub checkDate {
     $end ||= 999999999999999;
 
     # Get the present day and hour
-    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) =
-      gmtime(time);
-    $year += 1900;
-    $mon  += 1;
-    $mon  = "0" . $mon  if ( $mon < 10 );
-    $mday = "0" . $mday if ( $mday < 10 );
-    $hour = "0" . $hour if ( $hour < 10 );
-    $min  = "0" . $min  if ( $min < 10 );
-    $sec  = "0" . $sec  if ( $sec < 10 );
-
-    my $date = $year . $mon . $mday . $hour . $min . $sec;
+    my $date = &date;
 
     return 1 if ( ( $date >= $start ) and ( $date <= $end ) );
     return 0;
@@ -116,13 +126,13 @@ sub checkDate {
 # @param password User password
 # @return Authorization header content
 sub basic {
-    my ( $login, $password ) = splice @_;
+    my ( $login, $password ) = @_;
 
     # UTF-8 strings should be ISO encoded
     $login    = &unicode2iso($login);
     $password = &unicode2iso($password);
 
-    return "Basic " . encode_base64( $login . ":" . $password, '' );
+    return "Basic " . encode_base64( $login . ":" . $password );
 }
 
 ## @function string unicode2iso(string string)
@@ -130,7 +140,7 @@ sub basic {
 # @param string UTF-8 string
 # @return ISO string
 sub unicode2iso {
-    my ($string) = splice @_;
+    my ($string) = @_;
 
     return encode( "iso-8859-1", decode( "utf-8", $string ) );
 }
@@ -140,7 +150,7 @@ sub unicode2iso {
 # @param string ISO string
 # @return UTF-8 string
 sub iso2unicode {
-    my ($string) = splice @_;
+    my ($string) = @_;
 
     return encode( "utf-8", decode( "iso-8859-1", $string ) );
 }
@@ -218,9 +228,9 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =over
 
-=item Copyright (C) 2009, 2010 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+=item Copyright (C) 2009-2010 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
-=item Copyright (C) 2009, 2010, 2012 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+=item Copyright (C) 2009-2016 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
 =back
 
